@@ -54,7 +54,6 @@ import java.util.Hashtable;
 import org.osgi.service.http.HttpService;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.config.ForcedReloadable;
-import com.openexchange.config.Interests;
 import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.context.ContextService;
 import com.openexchange.database.DatabaseService;
@@ -100,11 +99,7 @@ public class SpamExpertsActivator extends HousekeepingActivator {
             public void reloadConfiguration(ConfigurationService configService) {
                 config.clearCache();
             }
-
-            @Override
-            public Interests getInterests() {
-                return null;
-            }
+            
         });
 
         String alias = getService(ConfigurationService.class).getProperty("com.openexchange.custom.spamexperts.panel_servlet", "/ajax/spamexperts/panel").trim();
